@@ -216,7 +216,12 @@ class CalibrationModel(Serializer):
             frequencies = np.squeeze(frequencies)
 
             self.frequencies_by_time_interpolator =\
-                interpolate.interp1d(calib_times_array, frequencies, axis=0)
+                interpolate.interp1d(
+                    calib_times_array,
+                    frequencies,
+                    bounds_error=False,
+                    fill_value=(frequencies[0], frequencies[-1]),
+                    axis=0)
 
         """
         Create the interpolator for getting a frequency by position and time
@@ -317,20 +322,32 @@ class CalibrationModel(Serializer):
         -------
         The corresponding frequency in Hz
         """
-        # Cannot execute if ndims are unequal
-        if np.ndim(time) is not np.ndim(position):
-            return None
+        # Convert time and position to array just in case it's not
+        time = np.array(time, ndmin=1)
+        position = np.array(position, ndmin=1)
 
-        shape_time = np.shape(time)
-        shape_position = np.shape(position)
+        # Cannot execute if ndims are unequal
+        # and we have more than one time value
+        if np.ndim(time) is not np.ndim(position):
+            if time.size == 1:
+                time = time[0] * np.ones(np.shape(position))
+            else:
+                return None
+
+        shape_time = np.array(np.shape(time))
+        shape_position = np.array(np.shape(position))
 
         # In case the arrays don't have the same shape,
         # we repeat the time array
-        if shape_time is not shape_position:
-            time = np.tile(
-                time,
-                (np.array(shape_position) - np.array(shape_time)) + 1
-            )
+        if (shape_time != shape_position).any():
+            # Only works if the respective time array dimension has size 1
+            if (shape_time[shape_time != shape_position] != 1).any():
+                return None
+            else:
+                time = np.tile(
+                    time,
+                    (np.array(shape_position) - np.array(shape_time)) + 1
+                )
 
         if self.frequency_by_time_interpolator is not None:
             return self.frequency_by_time_interpolator(time, position)
@@ -350,6 +367,8 @@ class CalibrationModel(Serializer):
 
         """
         spectrum = self.get_frequencies_by_time(time)
+        if spectrum is None:
+            return None
         f = interpolate.interp1d(spectrum, range(len(spectrum)))
         return f(frequencies)
 
